
/*
   wmch : A Window Manager CHooser
   Copyright (C) 1997  Dominique Rousseau <rousseau@neuronnexion.fr>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Parts from GUBI are Copyright (C) Tim Janik  <timj@psynet.net>
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>

#include	"ufunc.h"
#include	"widgets.h"

#include	"u_callback.h"
#include	"rcparser.h"
#include	"exec.h"

void wmch_add_button(GtkWindow * window, GtkContainer * box, rc_struct * info);
void toggle_button_click(GtkToggleButton * obj);

void sig_child(int sig);

void wmch_add_button(GtkWindow * window, GtkContainer * box, rc_struct * info)
{
	GtkPixmap *button_pixmap;
	GdkPixmap *the_pixmap;
	GdkBitmap *mask;
	GtkStyle *style;
	GtkToggleButton *new_button;
	GtkContainer *new_box;
	GtkLabel *label;

	/* New button */
	new_button = GTK_TOGGLE_BUTTON(gtk_toggle_button_new());
	/* Show it */
	gtk_widget_show(GTK_WIDGET(new_button));
	/* Get its style */
	style = gtk_widget_get_style(GTK_WIDGET(new_button));

	/* Create pixmap if pixmap name != NULL */
	if ((info->pixmap != NULL) && (strlen(info->pixmap) > 0)) {
		/* Create Gdk Pixmap */
		the_pixmap = gdk_pixmap_create_from_xpm(GTK_WIDGET(window)->window, &mask, &style->bg[GTK_STATE_NORMAL], info->pixmap);
		/* Create corresponding Gtk pixmap */
		button_pixmap = GTK_PIXMAP(gtk_pixmap_new(the_pixmap, mask));
		/* Show it */
		gtk_widget_show(GTK_WIDGET(button_pixmap));
	} else {
		button_pixmap = NULL;
	}

	/* Create Label != NULL */
	if ((info->label != NULL) && (strlen(info->label) > 0)) {
		/* Create Label */
		label = GTK_LABEL(gtk_label_new(info->label));
		/* Show it */
		gtk_widget_show(GTK_WIDGET(label));
	} else {
		label = NULL;
	}

	/* New VBox homogenous=FALSE, spacing = 5 */
	new_box = GTK_CONTAINER(gtk_vbox_new(FALSE, 5));
	/* Show it */
	gtk_widget_show(GTK_WIDGET(new_box));

	/* Put Pixmap in Box */
	if (button_pixmap != NULL)
		gtk_container_add(GTK_CONTAINER(new_box), GTK_WIDGET(button_pixmap));
	/* Put Label iin box */
	if (label != NULL)
		gtk_container_add(GTK_CONTAINER(new_box), GTK_WIDGET(label));
	/* Put Box in Button */
	gtk_container_add(GTK_CONTAINER(new_button), GTK_WIDGET(new_box));
	/* Put Button in ButtonBox */
	gtk_container_add(box, GTK_WIDGET(new_button));

	/* Set user data of new button to rc_struct */
	gtk_object_set_user_data(GTK_OBJECT(new_button), info);
	/* Set event handler */
	gtk_signal_connect(GTK_OBJECT(new_button), "button_release_event", GTK_SIGNAL_FUNC(toggle_button_click), new_button);
}

#define	RC_FILE	".wmchrc"

void wm_chooser_configure(void)
{
	GtkContainer *button_box;
	GtkWindow *window;
	rc_struct *tab;
	int count, i;
	char rc_file[256];

	button_box = GTK_CONTAINER(Button_Box->widget);
	window = GTK_WINDOW(Wm_Chooser->widget);

	/* Parse config file */
	count = 0;
	sprintf(rc_file, "%s/%s", getenv("HOME"), RC_FILE);
	wmch_rcparse(rc_file, &tab, &count);

	/* Error if rc file is empty */
	if (count == 0) {
		printf("Nothing in %s\n", rc_file);
		gtk_exit(1);
	}
	/* Add buttons */
	for (i = 0; i < count; i++) {
		wmch_add_button(window, button_box, (tab + i));
	}

	/* Disbale 'Configure' button */
	gtk_widget_set_sensitive (GTK_WIDGET (Config_Button->widget), FALSE);
}


void toggle_button_click(GtkToggleButton * obj)
{
	GList *list;
	GtkContainer *button_box;
	GtkToggleButton *the_button;

	/* Is this button already active */
	if (!obj->active) {
		/* Get pointer to ButonBox */
		button_box = GTK_CONTAINER(Button_Box->widget);

		/* Get list of children */
		list = g_list_first(gtk_container_children(button_box));

		while (list) {
			the_button = GTK_TOGGLE_BUTTON(list->data);
			gtk_toggle_button_set_state(the_button, FALSE);
			list = list->next;
		}
	} else {
		gtk_toggle_button_set_state(obj, TRUE);
		gtk_signal_emit_by_name(GTK_OBJECT(obj), "clicked");
	}

}

void sig_child(int sig)
{
	GtkWindow *window;

	window = GTK_WINDOW(Wm_Chooser->widget);
	gtk_widget_show(GTK_WIDGET(window));
}

void go_button_click(void)
{
	GtkContainer *button_box;
	GtkWindow *window;
	GList *list;
	GtkToggleButton *active_button;
	rc_struct *info;
	pid_t son_pid;
	char **argl;
	int argn;
	char t_command[256];

	/* Get ButtonBox pointer */
	button_box = GTK_CONTAINER(Button_Box->widget);
	/* Get children list */
	list = g_list_first(gtk_container_children(button_box));

	/* Get Window pointer */
	window = GTK_WINDOW(Wm_Chooser->widget);

	/* Search active button */
	while (list) {
		active_button = list->data;
		if (active_button->active)
			break;
		list = list->next;
	}

	if (!list) {
		printf("Warning : no actvie button\n");
		return;
	}
	/* Get user data */
	info = gtk_object_get_user_data(GTK_OBJECT(active_button));
	g_return_if_fail(info != NULL);

	strcpy(t_command, info->command);
	dispatch_cmd(t_command, &argl, &argn);

	switch (son_pid = fork()) {
	case 0:					/* In son */
		/* Clean up GTK */
		gtk_main_quit();
		/* Launch cmd */
		execvp(argl[0], argl);
		/* If any error in exec */
		perror("exec");
		exit(1);
		break;

	case -1:					/* fork failed */
		perror("fork");
		break;

	default:					/* In Dad */
		/* Hide window */
		gtk_widget_hide(GTK_WIDGET(window));

		/* Signal to handle child death */
		signal(SIGCHLD, sig_child);
		break;
	}

}
